/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */

/*
 * Copyright (C) 2000 Eazel, Inc
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 * 
 * Author: Andrew Burr <burra@colorado.edu>
 */

#include "nautilus-rpm-view.h"

#include <bonobo/bonobo-i18n.h>
#include <gtk/gtkframe.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtklabel.h>
#include <libgnome/gnome-macros.h>

#include <string.h>
#include <fcntl.h>
#include <rpmlib.h>

struct NautilusRpmViewDetails
{
  char *location;
  GtkWidget *label;
  GtkWidget *frame;
  GtkWidget *vbox;
  GtkWidget *name;
};

BONOBO_CLASS_BOILERPLATE (NautilusRpmView, nautilus_rpm_view,
			  NautilusView, NAUTILUS_TYPE_VIEW)
     static void nautilus_rpm_view_finalize (GObject * object)
{
  NautilusRpmView *view;

  view = NAUTILUS_RPM_VIEW (object);

  g_free (view->details->location);
  g_free (view->details);

  GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
}

static void
load_rpm_location (NautilusRpmView * view, const char *location)
{
  char *name;
  int stat;
  FD_t fd;
#ifdef RPMCALLS
  HeaderIterator iter;
  Header h, sig;
  int_32 itertag, type, count;
  void *p = NULL;
#endif

  g_assert (NAUTILUS_IS_RPM_VIEW (view));
  g_assert (location != NULL);

  g_free (view->details->location);
  view->details->location = g_strdup (location);

#ifdef RPMCALLS
  if ((fd = fdOpen (view->details->location, O_RDONLY, 0644)) == NULL)
    {
      printf ("OH SHIT 1\n");
    }

  if (rpmReadPackageInfo (fd, &sig, &h))
    {
      printf ("OH SHIT 2\n");
    }

  iter = headerInitIterator (h);
  while (headerNextIterator (iter, &itertag, &type, (hPTR_t *) & p, &count))
    {
      switch (itertag)
	{
	case RPMTAG_NAME:
	  if (type == RPM_STRING_TYPE)
	    {
	      printf ("NAME: %s\n", (char *) p);
	      name = strdup ((char *) p);
	      printf ("NAME: %s\n", name);
	      name =
		g_strdup_printf ("<span font_desc=\"Sans Bold 12\">%s</span>",
				 (char *) p);
	      printf ("NAME: %s\n", name);
	    }
	  break;
	case RPMTAG_SUMMARY:
	  break;
	}

      /* rpmlib allocates a buffer to return these two values... */
      if (type == RPM_STRING_ARRAY_TYPE || type == RPM_I18NSTRING_TYPE)
	{
	  free (p);
	}
    }
  headerFreeIterator (iter);
  headerFree (h);
  fdClose (fd);
#endif

  name = g_strdup_printf ("<b>%s</b>", location);
  gtk_label_set_text (GTK_LABEL (view->details->name), name);
  gtk_label_set_use_markup (GTK_LABEL (view->details->name), TRUE);
  g_free (name);
}

static void
rpm_load_location_callback (NautilusView * nautilus_view,
			    const char *location, gpointer user_data)
{
  NautilusRpmView *view;

  g_assert (NAUTILUS_IS_VIEW (nautilus_view));
  g_assert (location != NULL);

  view = NAUTILUS_RPM_VIEW (nautilus_view);

  nautilus_view_report_load_underway (nautilus_view);

  load_rpm_location (view, location);

  nautilus_view_report_load_complete (nautilus_view);
}

static void
bonobo_rpm_callback (BonoboUIComponent * ui,
		     gpointer user_data, const char *verb)
{
  NautilusRpmView *view;
  char *action;

  g_assert (BONOBO_IS_UI_COMPONENT (ui));
  g_assert (verb != NULL);

  view = NAUTILUS_RPM_VIEW (user_data);

  if (strcmp (verb, "Rpm Menu Item") == 0)
    {
      action = g_strdup_printf (_("%s\n\nRpm menu item."),
				view->details->location);
    }
  else
    {
      g_assert (strcmp (verb, "Rpm Dock Item") == 0);
      action = g_strdup_printf (_("%s\nRpm toolbar button."),
				view->details->location);
    }

  gtk_label_set_text (GTK_LABEL (view->details->label), action);
  g_free (action);
}

static void
rpm_merge_bonobo_items_callback (BonoboControl * control,
				 gboolean state, gpointer user_data)
{
  NautilusRpmView *view;
  BonoboUIComponent *ui_component;
  BonoboUIVerb verbs[] = {
    BONOBO_UI_VERB ("Rpm Menu Item", bonobo_rpm_callback),
    BONOBO_UI_VERB ("Rpm Dock Item", bonobo_rpm_callback),
    BONOBO_UI_VERB_END
  };

  g_assert (BONOBO_IS_CONTROL (control));

  view = NAUTILUS_RPM_VIEW (user_data);

  if (state)
    {
      ui_component = nautilus_view_set_up_ui (NAUTILUS_VIEW (view),
					      DATADIR,
					      "nautilus-rpm-view-ui.xml",
					      "nautilus-rpm-view");

      bonobo_ui_component_add_verb_list_with_data (ui_component, verbs, view);
    }
}

static void
nautilus_rpm_view_class_init (NautilusRpmViewClass * class)
{
  G_OBJECT_CLASS (class)->finalize = nautilus_rpm_view_finalize;
}

static void
nautilus_rpm_view_instance_init (NautilusRpmView * view)
{
  view->details = g_new0 (NautilusRpmViewDetails, 1);

  view->details->frame = gtk_frame_new (NULL);
  gtk_container_set_border_width (GTK_CONTAINER (view->details->frame), 4);
  gtk_frame_set_shadow_type (GTK_FRAME (view->details->frame),
			     GTK_SHADOW_NONE);

  view->details->vbox = gtk_vbox_new (FALSE, 4);
  gtk_container_add (GTK_CONTAINER (view->details->frame),
		     view->details->vbox);

  view->details->name = gtk_label_new (_("no file"));
  gtk_label_set_use_markup (GTK_LABEL (view->details->name), TRUE);
  gtk_box_pack_start (GTK_BOX (view->details->vbox), view->details->name,
		      FALSE, FALSE, 0);

  view->details->label =
    gtk_label_new (_("<span font_desc=\"Sans 20\">(none)</span>"));
  gtk_label_set_use_markup (GTK_LABEL (view->details->label), TRUE);
  gtk_box_pack_start (GTK_BOX (view->details->vbox), view->details->label,
		      FALSE, FALSE, 0);

  gtk_widget_show_all (view->details->frame);

  nautilus_view_construct (NAUTILUS_VIEW (view), view->details->frame);

  g_signal_connect (view,
		    "load_location",
		    G_CALLBACK (rpm_load_location_callback), NULL);

  g_signal_connect (nautilus_view_get_bonobo_control (NAUTILUS_VIEW (view)),
		    "activate",
		    G_CALLBACK (rpm_merge_bonobo_items_callback), view);
}
